package net.zenrindatacom.geofenceapp_sample.activity.area;

import android.app.AlertDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import androidx.appcompat.app.AppCompatActivity;

import net.zenrindatacom.geofenceapp_sample.activity.general.DialogUtil;
import net.zenrindatacom.geofenceapp_sample.activity.general.HashInformationActivity;
import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.geofence.GeoFenceWrapperManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * エリア情報管理画面
 */
public class AreaInformationMngActivity extends AppCompatActivity {

    /** ログのTAG */
    private static final String TAG = "AreaInformationMngActivity";

    /**
     * onCreate
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.area_information_mng);
    }

    /**
     * 画面内のボタンがタップされた
     *
     * @param view タップされたアイテム
     */
    public void onClick(View view) {

        //2度押し防止
        Button button = (Button)view;
        button.setEnabled(false);
        new Handler(Looper.getMainLooper()).postDelayed(() -> button.setEnabled(true), 1000);

        int viewId = view.getId();

        if(viewId == R.id.btnAddAreaInformation) {
            //エリア情報を追加
            Intent intent = new Intent(this, AreaInformationAddActivity.class);
            startActivity(intent);

        } else if(viewId == R.id.btnDeleteAreaInformation) {
            //エリア情報を削除
            removeAreaInformation();

        } else if(viewId == R.id.btnGetAreaInformation) {
            //エリア情報を取得
            getAreaInformationList();
        }
    }

    /**
     * エリア情報を削除
     */
    private void removeAreaInformation() {

        //エリアIDの入力フィールド
        final EditText editTextAreaId = new EditText(this);

        //確認ダイアログの表示
        new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_info)
                .setTitle(R.string.dlg_title_confirm_delete_area_information)
                .setMessage(R.string.dlg_msg_confirm_delete_area_information)
                .setView(editTextAreaId)
                .setPositiveButton(R.string.dlg_btn_ok, (dialog, whichButton) -> {
                    //OKボタンをタップ

                    String inputAreaId = editTextAreaId.getText().toString();
                    Log.d(TAG, "Input Area Id=" + inputAreaId);

                    ArrayList<String> inputAreaIdList = new ArrayList<>();
                    if (inputAreaId.length() > 0) {
                        inputAreaIdList.add(inputAreaId);
                    }

                    //ジオフェンスライブラリからエリア情報を削除する
                    List<String> resultAreaIdList = GeoFenceWrapperManager.getGeoFenceManager().removeAreaInformation(inputAreaIdList);

                    if (resultAreaIdList != null && resultAreaIdList.size() > 0) {
                        //削除したものが存在する
                        DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_delete_success);
                    } else {
                        //削除したものが存在しない
                        DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_delete_fail);
                    }
                })
                .setNegativeButton(R.string.dlg_btn_cancel, null)
                .show();
    }

    /**
     * エリア情報を取得
     */
    private void getAreaInformationList() {

        //エリアIDの入力フィールド
        final EditText editTextAreaId = new EditText(this);

        //確認ダイアログの表示
        new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_info)
                .setTitle(R.string.dlg_title_confirm_get_area_information)
                .setMessage(R.string.dlg_msg_confirm_get_area_information)
                .setView(editTextAreaId)
                .setPositiveButton(R.string.dlg_btn_ok, (dialog, whichButton) -> {
                    //OKボタンをタップ

                    String inputAreaId = editTextAreaId.getText().toString();
                    Log.d(TAG, "Input Area Id=" + inputAreaId);

                    ArrayList<String> inputAreaIdList = new ArrayList<>();
                    if (inputAreaId.length() > 0) {
                        inputAreaIdList.add(inputAreaId);
                    }

                    //ジオフェンスライブラリからエリア情報を取得する
                    HashMap<String, Object> resultMap = (HashMap<String, Object>) GeoFenceWrapperManager.getGeoFenceManager().areaInformationList(inputAreaIdList);

                    if (resultMap.size() > 0) {
                        //エリア情報が存在する場合は「エリア情報表示画面」に表示
                        Intent intent = new Intent(this, HashInformationActivity.class);
                        //画面タイトル
                        intent.putExtra(HashInformationActivity.INTENT_KEY_TITLE, getString(R.string.area_information_title));
                        //表示データ
                        intent.putExtra(HashInformationActivity.INTENT_KEY_HASHMAP_INFO, resultMap);

                        startActivity(intent);

                    } else {
                        //エリア情報が存在しない場合
                        DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_area_not_found);
                    }
                })
                .setNegativeButton(R.string.dlg_btn_cancel, null)
                .show();
    }
}
