package net.zenrindatacom.geofenceapp_sample.activity.general;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;

import androidx.annotation.StringRes;

import net.zenrindatacom.geofenceapp_sample.R;

/**
 * ダイアログユーティリティ
 */
public class DialogUtil {
    /**
     * Alertダイアログ表示
     * @param ctx 呼び元画面のコンテキスト
     * @param messageId メッセージID(string.xmlで定義しているもの)
     */
    public static void showAlert(Context ctx, @StringRes int messageId){
        _showAlert(ctx, null, ctx.getString(messageId));
    }

    /**
     * Alertダイアログ表示
     * @param ctx 呼び元画面のコンテキスト
     * @param titleId タイトルID(string.xmlで定義しているもの)
     * @param messageId メッセージID(string.xmlで定義しているもの)
     */
    public static void showAlert(Context ctx, @StringRes int titleId, @StringRes int messageId){
        _showAlert(ctx, ctx.getString(titleId), ctx.getString(messageId));
    }

    /**
     * Alertダイアログ表示
     * @param ctx 呼び元画面のコンテキスト
     * @param title タイトル
     * @param message メッセージ
     */
    public static void showAlert(Context ctx, String title, String message){
        _showAlert(ctx, title, message);
    }

    /**
     * Alertダイアログ表示
     * @param ctx 呼び元画面のコンテキスト
     * @param title タイトル
     * @param message メッセージ
     */
    private static void _showAlert(Context ctx, String title, String message){
        AlertDialog.Builder alert = new AlertDialog.Builder(ctx);
        if(title != null) {
            alert.setTitle(title);
        }
        alert.setMessage(message);
        alert.setPositiveButton(R.string.dlg_btn_ok, null);
        alert.show();
    }

    private static ProgressDialog progressDialog;

    /**
     * プログレスダイアログ表示
     * @param ctx 呼び元画面のコンテキスト
     * @param messageId メッセージID(string.xmlで定義しているもの)
     */
    public static void showProgress(Context ctx, @StringRes int messageId) {
        progressDialog = new ProgressDialog(ctx);
        progressDialog.setMessage(ctx.getString(messageId));
        progressDialog.setCancelable(false);
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.show();
    }

    /**
     * 表示中しているプログレスダイアログ上のメッセージを更新
     * @param ctx 呼び元画面のコンテキスト
     * @param messageId メッセージID(string.xmlで定義しているもの)
     */
    public static void updateProgress(Context ctx, @StringRes int messageId) {
        progressDialog.setMessage(ctx.getString(messageId));
    }

    /**
     * プログレスダイアログ非表示
     */
    public static void hideProgress() {
        progressDialog.dismiss();
    }
}
