package net.zenrindatacom.geofenceapp_sample.activity.main;

import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;


import androidx.annotation.StringRes;
import androidx.appcompat.app.AppCompatActivity;

import net.zenrindatacom.geofenceapp_sample.activity.area.AreaInformationMngActivity;
import net.zenrindatacom.geofenceapp_sample.activity.check.LocationLibCheckActivity;
import net.zenrindatacom.geofenceapp_sample.activity.check.ManualCheckActivity;
import net.zenrindatacom.geofenceapp_sample.activity.general.CustomAsyncTask;
import net.zenrindatacom.geofenceapp_sample.activity.general.DialogUtil;
import net.zenrindatacom.geofenceapp_sample.activity.notification.NotificationMngActivity;
import net.zenrindatacom.geofenceapp_sample.activity.setting.AppSettingActivity;
import net.zenrindatacom.geofenceapp_sample.activity.setting.GeoFenceLibSettingActivity;
import net.zenrindatacom.geofenceapp_sample.activity.setting.LocationLibSettingActivity;
import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.geofence.GeoFenceWrapperException;
import net.zenrindatacom.geofenceapp_sample.preference.AppPreference;
import net.zenrindatacom.geofenceapp_sample.preference.GeoFenceLibPreference;
import net.zenrindatacom.geofenceapp_sample.preference.LocationLibPreference;
import net.zenrindatacom.geofenceapp_sample.geofence.GeoFenceWrapperManager;
import net.zenrindatacom.geofencelib.GeoFenceManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * メイン画面
 *
 * @author ZENRIN DataCom CO., LTD. All Rights Reserved.
 */
public class MainActivity extends AppCompatActivity {

    /**
     * onCreate
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //「ジオフェンスライブラリ設定」プリファレンス初期化（初回起動時のみ初期化される）
        GeoFenceLibPreference.getInstance().init();

        //「アプリ設定」プリファレンス初期化（初回起動時のみ初期化される）
        AppPreference.getInstance().init();

        //「位置測位ライブラリ設定」プリファレンス初期化（初回起動時のみ初期化される）
        LocationLibPreference.getInstance().init();
    }

    /**
     * オプションメニューの生成
     *
     * @param menu メニュー
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    /**
     * メニュー内のボタンがタップされた
     *
     * @param item タップされたメニューアイテム
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        Intent intent = null;
        int itemId = item.getItemId();

        if(itemId == R.id.btnShowGeofenceLibSetting) {
            //ジオフェンスライブラリ設定画面を表示
            intent = new Intent(this, GeoFenceLibSettingActivity.class);
        } else if(itemId == R.id.btnShowAppSetting){
            //アプリ設定画面を表示
            intent = new Intent(this, AppSettingActivity.class);
        } else if(itemId == R.id.btnShowLocationLibSetting) {
            //位置測位ライブラリ設定画面を表示
            intent = new Intent(this, LocationLibSettingActivity.class);
        }

        if (intent != null) {
            startActivity(intent);
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * 画面内のボタンがタップされた
     *
     * @param view タップされたアイテム
     */
    public void onClick(View view) {

        //2度押し防止
        Button button = (Button)view;
        button.setEnabled(false);
        new Handler(Looper.getMainLooper()).postDelayed(() -> button.setEnabled(true), 1000);

        Intent intent = null;
        int viewId = view.getId();

        if(viewId == R.id.btnGetAreaFromServer) {
            //サーバからエリアを反映
            new AreaReflectionSequence(this).doSequence();

        } else if(viewId == R.id.btnManageArea) {
            //エリア情報管理
            intent = new Intent(this, AreaInformationMngActivity.class);

        } else if(viewId == R.id.btnManageNotification) {
            //通知条件管理
            intent = new Intent(this, NotificationMngActivity.class);

        } else if(viewId == R.id.btnResetAreaAndNotification) {
            //エリア情報 / 通知条件リセット
            resetAreaAndNotification();

        }  else if(viewId == R.id.btnCheckByManual) {
            //手入力で確認
            intent = new Intent(this, ManualCheckActivity.class);

        } else if(viewId == R.id.btnCheckByLocationLib) {
            //位置測位ライブラリで確認
            intent = new Intent(this, LocationLibCheckActivity.class);

        } else if(viewId == R.id.btnGetVersion) {
            //バージョン取得
            String version = String.format(getString(R.string.toast_geofence_lib_version), GeoFenceManager.libraryVersion());
            Toast.makeText(this, version, Toast.LENGTH_SHORT).show();
        }

        if (intent != null) {
            startActivity(intent);
        }
    }

    /**
     * エリア情報 / 通知条件リセット
     * 　ジオフェンスライブラリの「通知条件、エリア情報」を全て削除する
     */
    private void resetAreaAndNotification() {
        new AlertDialog.Builder(this)
                .setTitle(R.string.dlg_title_confirm_reset_all_information)
                .setMessage(R.string.dlg_msg_confirm_reset_all_information)
                .setPositiveButton(R.string.dlg_btn_ok, (dialog, which) -> {
                    //OKボタンをタップ

                    //ジオフェンスライブラリの「通知条件、エリア情報」を全て削除
                    GeoFenceWrapperManager.getGeoFenceManager().clearAllInformation();

                    //削除完了ダイアログを表示
                    DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_reset_all_information_done);
                })
                .setNegativeButton(R.string.dlg_btn_cancel, null)
                .show();
    }

    /**
     * サーバからジオフェンスライブラリにエリアを反映するシーケンス
     *
     * 以下の一連の処理を行う
     *  (1)サーバからエリア更新日時を取得する（処理中は進行中ダイアログを表示する）
     *  (2)サーバのエリアをジオフェンスライブラリに反映するか確認ダイアログで確認する
     *  (3)上記(2)で「はい」がタップされた時
     *  　(a)サーバからエリアを取得する（処理中は進行中ダイアログを表示する）
     *  　(b)ジオフェンスライブラリ上のエリアと通知条件をクリアする
     *  　(c)取得したエリアをジオフェンスライブラリに追加する
     *  　(d)取得したエリア毎にINとOUTの通知条件をジオフェンスライブラリに追加する
     *  　(e)完了ダイアログを表示する
     *  (4)上記(2)で「いいえ」がタップされた時
     *  　・処理終了
     */
    private class AreaReflectionSequence {

        /** コンテキスト */
        private final Context ctx;

        /** サーバ上の「エリア更新日時」 */
        private String serverAreaUpdateTime;

        /** コンストラクタ
         * @param ctx コンテキスト
         */
        public AreaReflectionSequence(Context ctx) {
            this.ctx = ctx;
        }

        /** シーケンス開始 */
        public void doSequence() {
            getAreaUpdateTimeFromServer();
        }

        /**
         * サーバの「エリア更新日時」を取得する
         */
        private void getAreaUpdateTimeFromServer() {

            //処理中ダイアログ(確認中)を表示
            DialogUtil.showProgress(ctx, R.string.dlg_msg_progress_confirm);

            //サーバから「エリア更新日時」を取得する（非同期）
            new CustomAsyncTask() {
                @Override
                public String doAsyncTask() throws GeoFenceWrapperException {
                    //サーバから「エリア更新日時」を取得
                    return GeoFenceWrapperManager.getInstance().getAreaUpdateTimeFromServer();
                }

                @Override
                public void onSuccess(Object result) {
                    //処理中ダイアログを非表示
                    DialogUtil.hideProgress();
                    //サーバ上のエリア更新日時
                    serverAreaUpdateTime = (String)result;
                    //エリア情報の反映確認
                    confirmReflectArea();
                }

                @Override
                public void onException(Exception e) {
                    //処理中ダイアログを非表示
                    DialogUtil.hideProgress();
                    //例外エラーを表示する
                    showGeoFenceWrapperException((GeoFenceWrapperException)e);
                }
            }.execute();
        }

        /**
         * エリアの反映確認
         */
        private void confirmReflectArea() {

            //ローカル上のエリア更新日時
            String localAreaUpdateTime = AppPreference.getInstance().getAreaUpdateTime();
            if(TextUtils.isEmpty(localAreaUpdateTime)) {
                //ローカル上に「エリア更新日時」が存在しない場合は、ダイアログ上に「なし」を表示する
                localAreaUpdateTime = ctx.getString(R.string.dlg_msg_confirm_area_reflection_data_not_found);
            }

            String message = String.format(getString(R.string.dlg_msg_confirm_area_reflection), serverAreaUpdateTime, localAreaUpdateTime);

            //サーバ上のエリアをジオフェンスライブラリに反映するか確認する
            new AlertDialog.Builder(ctx)
                    .setIcon(android.R.drawable.ic_dialog_info)
                    .setTitle(R.string.dlg_title_confirm_reflect_area_information)
                    .setMessage(message)
                    .setPositiveButton(R.string.dlg_btn_yes, (dialog, whichButton) -> {
                        //「はい」（反映する）
                        getAreaFromServer();
                    })
                    .setNegativeButton(R.string.dlg_btn_no, (dialog, whichButton) -> {
                        //「いいえ」（反映しない）
                    })
                    .show();
        }

        /**
         * サーバからエリアを取得する
         */
        private void getAreaFromServer() {

            //処理中ダイアログ(取得中)を表示
            DialogUtil.showProgress(ctx, R.string.dlg_msg_progress_get);

            //サーバからエリアを取得する（非同期）
            new CustomAsyncTask() {
                @Override
                public ArrayList<GeoFenceWrapperManager.AreaInfo> doAsyncTask() throws GeoFenceWrapperException {
                    //サーバから「エリア」を取得
                    return GeoFenceWrapperManager.getInstance().getAreaFromServer();
                }

                @Override
                public void onSuccess(Object result) {
                    @SuppressWarnings("unchecked")
                    ArrayList<GeoFenceWrapperManager.AreaInfo> areaList = (ArrayList<GeoFenceWrapperManager.AreaInfo>)result;
                    //サーバ上のエリアをジオフェンスライブラリに反映
                    reflectToGeoFenceLib(areaList);
                }

                @Override
                public void onException(Exception e) {
                    //処理中ダイアログを非表示
                    DialogUtil.hideProgress();
                    //例外エラーを表示する
                    showGeoFenceWrapperException((GeoFenceWrapperException)e);
                }
            }.execute();
        }

        /**
         * サーバのエリアをジオフェンスライブラリに反映する
         */
        private void reflectToGeoFenceLib(ArrayList<GeoFenceWrapperManager.AreaInfo> serverAreaInfoList) {

            //処理中ダイアログのメッセージ部を更新（更新中）
            DialogUtil.updateProgress(ctx, R.string.dlg_msg_progress_update);

            //サーバのエリアをジオフェンスライブラリに反映する（非同期）
            new CustomAsyncTask() {
                @Override
                public Object doAsyncTask() throws Exception {

                    GeoFenceManager geoFenceMng = GeoFenceWrapperManager.getGeoFenceManager();

                    //ジオフェンスライブラリの「通知条件、エリア情報」を全て削除
                    geoFenceMng.clearAllInformation();

                    //ライブラリに設定するエリア情報リスト
                    ArrayList<HashMap<String, Object>> areaInfoList = new ArrayList<>();

                    //ライブラリに設定する通知条件リスト
                    ArrayList<HashMap<String, Object>> notificationInfoList = new ArrayList<>();

                    //通知条件ID（1からの連番で設定する）
                    int notificationId = 1;

                    //サーバから取得したエリア毎
                    for(GeoFenceWrapperManager.AreaInfo serverAreaInfo : serverAreaInfoList) {

                        //ライブラリに設定する「エリアID」＝ サーバ「エリアグループID」#サーバ「エリアID」
                        String libAreaId = serverAreaInfo.getAreaGroupId() + "#" + serverAreaInfo.getAreaId();

                        //==== ライブラリに設定するエリア情報 ====
                        HashMap<String, Object> areaInfo = new HashMap<>();
                        {
                            //エリアID
                            areaInfo.put(GeoFenceManager.RESPONSE_KEY_AID, libAreaId);
                            //ポリゴン
                            String libAreaShape = serverAreaInfo.getAreaShape();
                            areaInfo.put(GeoFenceManager.RESPONSE_KEY_GINF, libAreaShape);
                            //IN確定回数閾値
                            areaInfo.put(GeoFenceManager.RESPONSE_KEY_INFNUM, 1);
                            //OUT確定回数閾値
                            areaInfo.put(GeoFenceManager.RESPONSE_KEY_OUTFNUM, 1);

                            areaInfoList.add(areaInfo);
                        }

                        //==== ライブラリに設定する通知条件（通知タイミング:IN） ====
                        HashMap<String, Object> notificationInfo = new HashMap<>();
                        {
                            //通知条件ID
                            notificationInfo.put(GeoFenceManager.RESPONSE_KEY_NID, notificationId++);
                            //エリアID
                            notificationInfo.put(GeoFenceManager.RESPONSE_KEY_AID, libAreaId);
                            //通知タイミング
                            notificationInfo.put(GeoFenceManager.RESPONSE_KEY_NTMG, 1);

                            notificationInfoList.add(notificationInfo);
                        }

                        //==== ライブラリに設定する通知条件（通知タイミング:OUT） ====
                        notificationInfo = new HashMap<>();
                        {
                            //通知条件ID
                            notificationInfo.put(GeoFenceManager.RESPONSE_KEY_NID, notificationId++);
                            //エリアID
                            notificationInfo.put(GeoFenceManager.RESPONSE_KEY_AID, libAreaId);
                            //通知タイミング
                            notificationInfo.put(GeoFenceManager.RESPONSE_KEY_NTMG, 2);

                            notificationInfoList.add(notificationInfo);
                        }
                    }

                    //ジオフェンスライブラリにエリア（GeoJSON形式）を追加
                    List<?> failList = geoFenceMng.addAreaInformationForGeoJson(areaInfoList);

                    if (failList == null || failList.size() > 0) {
                        //追加失敗
                        throw new Exception(ctx.getString(R.string.dlg_msg_alert_geofence_lib_reflect_area_fail));
                    }

                    //ジオフェンスライブラリに通知条件を追加
                    failList = GeoFenceWrapperManager.getGeoFenceManager().addNotificationConditionInformation(notificationInfoList);

                    if (failList == null || failList.size() > 0) {
                        //追加失敗
                        throw new Exception(ctx.getString(R.string.dlg_msg_alert_geofence_lib_reflect_notification_fail));
                    }

                    //サーバ上の「エリア更新日時」をプリファレンスに保存する
                    AppPreference.getInstance().setAreaUpdateTime(serverAreaUpdateTime);

                    return null;
                }

                @Override
                public void onSuccess(Object result) {
                    //処理中ダイアログを非表示
                    DialogUtil.hideProgress();
                    //ジオフェンスライブラリへの反映完了を表示
                    DialogUtil.showAlert(ctx, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_reflect_success);
                }

                @Override
                public void onException(Exception e) {
                    //処理中ダイアログを非表示
                    DialogUtil.hideProgress();
                    //例外エラーを表示する
                    DialogUtil.showAlert(ctx, ctx.getString(R.string.dlg_title_alert_geofence_lib), e.getMessage());
                }
            }.execute();
        }

        /**
         * GeoFenceWrapperManagerから通知された例外を表示する
         * @param ex GeoFenceWrapperManagerの例外
         */
        private void showGeoFenceWrapperException(GeoFenceWrapperException ex) {

            @StringRes int messageId;

            switch(ex.getErrorType()){
                case TOKEN_SERVER_TIMEOUT:
                    //トークン発行通信エラー(タイムアウト)
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_token_timeout;
                    break;
                case TOKEN_SERVER_AUTH_FAILED:
                    //トークン発行認証失敗
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_token_auth_failed;
                    break;
                case TOKEN_SERVER_ERROR:
                    //トークン発行サーバエラー(タイムアウトと認証失敗以外のエラー)
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_token_error;
                    break;
                case TOKEN_SERVER_LOCAL_ERROR:
                    //トークン発行サーバエラー(ローカル上のエラー)
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_token_local;
                    break;
                case AREA_SERVER_TIMEOUT:
                    //認証通信エラー(タイムアウト)
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_area_timeout;
                    break;
                case AREA_SERVER_ERROR:
                    //認証サーバエラー(タイムアウト以外)
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_area_error;
                    break;
                case AREA_SERVER_LOCAL_ERROR:
                    //認証サーバエラー(ローカル上のエラー)
                default:
                    messageId = R.string.dlg_msg_alert_app_auth_error_state_area_local;
                    break;
            }

            DialogUtil.showAlert(ctx, R.string.dlg_title_alert_get_server_area, messageId);
        }
    }
}
