package net.zenrindatacom.geofenceapp_sample.activity.notification;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import androidx.appcompat.app.AppCompatActivity;

import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.activity.general.DialogUtil;
import net.zenrindatacom.geofenceapp_sample.geofence.GeoFenceWrapperManager;
import net.zenrindatacom.geofencelib.GeoFenceManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 通知条件追加画面
 */
public class NotificationAddActivity extends AppCompatActivity {

    /** 入力フィールド：通知条件ID */
    private EditText m_EditTextNid;
    /** 入力フィールド：エリアID */
    private EditText m_EditTextAid;
    /** 入力フィールド：通知タイミング */
    private EditText m_EditTextNtmg;
    /** 入力フィールド：通知曜日 */
    private EditText m_EditTextNwk;
    /** 入力フィールド：通知開始時間 */
    private EditText m_EditTextNstm;
    /** 入力フィールド：通知終了時間 */
    private EditText m_EditTextNetm;
    /** 入力フィールド：通知開始期間 */
    private EditText m_EditTextNsvd;
    /** 入力フィールド：通知終了期間 */
    private EditText m_EditTextNevd;
    /** 入力フィールド：再通知禁止日数 */
    private EditText m_EditTextNotnd;

    /**
     * onCreate
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.notification_add);
        findView();
    }

    /**
     * 画面リソースの展開
     */
    private void findView() {
        m_EditTextNid = findViewById(R.id.editTextNid);
        m_EditTextAid = findViewById(R.id.editTextAid);
        m_EditTextNtmg = findViewById(R.id.editTextNtmg);
        m_EditTextNwk = findViewById(R.id.editTextNwk);
        m_EditTextNstm = findViewById(R.id.editTextNstm);
        m_EditTextNetm = findViewById(R.id.editTextNetm);
        m_EditTextNsvd = findViewById(R.id.editTextNsvd);
        m_EditTextNevd = findViewById(R.id.editTextNevd);
        m_EditTextNotnd = findViewById(R.id.editTextNotnd);
    }

    /**
     * 画面内のボタンがタップされた
     *
     * @param view タップされたアイテム
     */
    public void onClick(View view) {

        //2度押し防止
        Button button = (Button)view;
        button.setEnabled(false);
        new Handler(Looper.getMainLooper()).postDelayed(() -> button.setEnabled(true), 1000);

        if (view.getId() == R.id.btnAddNotification) {
            //追加
            addNotification();
        }
    }

    /**
     * ジオフェンスライブラリへ「通知条件」を追加する
     */
    private void addNotification() {

        HashMap<String, Object> information = new HashMap<>();

        //通知条件ID
        if (m_EditTextNid.length() > 0) {
            int parseValue;
            try {
                parseValue = Integer.parseInt(m_EditTextNid.getText().toString());
            } catch(NumberFormatException ex) {
                DialogUtil.showAlert(this, R.string.dlg_msg_alert_notification_id_format_error);
                return;
            }
            information.put(GeoFenceManager.RESPONSE_KEY_NID, parseValue);
        }

        //エリアID
        if (m_EditTextAid.length() > 0) {
            information.put(GeoFenceManager.RESPONSE_KEY_AID, m_EditTextAid.getText().toString());
        }

        //通知タイミング
        if (m_EditTextNtmg.length() > 0) {
            int parseValue;
            try {
                parseValue = Integer.parseInt(m_EditTextNtmg.getText().toString());
            } catch(NumberFormatException ex) {
                DialogUtil.showAlert(this, R.string.dlg_msg_alert_notification_timing_format_error);
                return;
            }
            information.put(GeoFenceManager.RESPONSE_KEY_NTMG, parseValue);
        }

        //通知曜日
        if (m_EditTextNwk.length() > 0) {
            information.put(GeoFenceManager.RESPONSE_KEY_NWK, m_EditTextNwk.getText().toString());
        }

        //通知開始時間
        if (m_EditTextNstm.length() > 0) {
            information.put(GeoFenceManager.RESPONSE_KEY_NSTM, m_EditTextNstm.getText().toString());
        }

        //通知終了時間
        if (m_EditTextNetm.length() > 0) {
            information.put(GeoFenceManager.RESPONSE_KEY_NETM, m_EditTextNetm.getText().toString());
        }

        //通知開始期間
        if (m_EditTextNsvd.length() > 0) {
            information.put(GeoFenceManager.RESPONSE_KEY_NSVD, m_EditTextNsvd.getText().toString());
        }

        //通知終了期間
        if (m_EditTextNevd.length() > 0) {
            information.put(GeoFenceManager.RESPONSE_KEY_NEVD, m_EditTextNevd.getText().toString());
        }

        //再通知禁止日数
        if (m_EditTextNotnd.length() > 0) {
            int parseValue;
            try {
                parseValue = Integer.parseInt(m_EditTextNotnd.getText().toString());
            } catch(NumberFormatException ex) {
                DialogUtil.showAlert(this, R.string.dlg_msg_alert_notification_cool_down_days_format_error);
                return;
            }
            information.put(GeoFenceManager.RESPONSE_KEY_NOTND, parseValue);
        }

        //1つ以上の項目が入力されているか？
        if (information.size() == 0) {
            DialogUtil.showAlert(this, R.string.dlg_msg_alert_no_entry);
            return;
        }

        ArrayList<HashMap<String, Object>> informationList = new ArrayList<>();
        informationList.add(information);

        //ジオフェンスライブラリに通知条件を追加
        List<?> resultList = GeoFenceWrapperManager.getGeoFenceManager().addNotificationConditionInformation(informationList);

        if (resultList == null || resultList.size() > 0) {
            //追加失敗
            DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_add_fail);
            return;
        }

        //追加成功
        DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_add_success);
    }
}
