package net.zenrindatacom.geofenceapp_sample.activity.setting;

import android.os.Bundle;
import android.util.Log;

import androidx.appcompat.app.AppCompatActivity;
import androidx.preference.DropDownPreference;
import androidx.preference.Preference;

import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.preference.GeoFenceLibPreference;

/**
 * ジオフェンスライブラリ設定画面
 *
 * @author N.Miyahara
 */
public class GeoFenceLibSettingActivity extends AppCompatActivity {

    /** ログのTAG */
    private static final String TAG = "GeoFenceLibSettingActivity";

    /**
     * onCreate
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getSupportFragmentManager().beginTransaction().replace(android.R.id.content, new GeoFenceLibSettingPreferenceFragment()).commit();
    }

    /**
     * 本画面のPreferenceFragmentクラス
     */
    public static class GeoFenceLibSettingPreferenceFragment extends SettingPreferenceFragment implements Preference.OnPreferenceClickListener {
        /**
         * 親クラスのonCreatePreferences()イベント発生時に呼び出される
         * 　本タイミングで初期化に必要な処理を行う。
         */
        @Override
        public void onCreateSettingPreference(String rootKey) {
            //対象とするプリファレンスファイル名をデフォルトから変更
            getPreferenceManager().setSharedPreferencesName(GeoFenceLibPreference.getInstance().getPreferenceName());
            //本設定画面と対になるPreferenceScreen設定ファイル
            setPreferencesFromResource(R.xml.geofence_lib_setting, rootKey);
            //タップ時のリスナー登録（「デフォルト設定に戻す」ボタン）
            Preference preference = findPreference(getString(R.string.prf_geofence_lib_key_returnToDefaultSetting));
            if(preference != null) {
                preference.setOnPreferenceClickListener(this);
            }
        }

        /**
         * PreferenceClickリスナー
         * 　PreferenceScreen上のボタンがタップされるとCALLされる
         *
         * @return イベントハンドリング結果
         */
        @Override
        public boolean onPreferenceClick(Preference preference) {

            Log.d(TAG, "Tap!! (" + preference.getKey() + ")");

            if(preference.getKey().equals(getString(R.string.prf_geofence_lib_key_returnToDefaultSetting))) {
                //「デフォルト設定に戻す」ボタンをタップ
                GeoFenceLibPreference.getInstance().returnToDefault();
                // ドロップダウンメニューの表示の初期化
                DropDownPreference dropDownPreference = findPreference(getString(R.string.prf_geofence_lib_key_auth_env));
                if (dropDownPreference != null) {
                    dropDownPreference.setValueIndex(0);
                }

                return true;
            }

            //処理なし
            return false;
        }
    }
}
