package net.zenrindatacom.geofenceapp_sample.geofence;

/**
 * GeoFenceWrapperManagerで発生する例外を集約した例外クラス
 *
 * @author N.Miyahara
 */
public class GeoFenceWrapperException extends Exception {
    public enum ErrorType {
        /** トークン発行通信エラー(タイムアウト) */
        TOKEN_SERVER_TIMEOUT,
        /** トークン発行認証失敗 */
        TOKEN_SERVER_AUTH_FAILED,
        /** トークン発行サーバエラー(タイムアウトと認証失敗以外のエラー) */
        TOKEN_SERVER_ERROR,
        /** トークン発行サーバエラー(ローカル上のエラー) */
        TOKEN_SERVER_LOCAL_ERROR,

        /** 認証通信エラー(タイムアウト) */
        AREA_SERVER_TIMEOUT,
        /** 認証サーバエラー(タイムアウト以外) */
        AREA_SERVER_ERROR,
        /** 認証サーバエラー(ローカル上のエラー) */
        AREA_SERVER_LOCAL_ERROR
    }

    /** エラータイプ */
    private final ErrorType errorType;

    public GeoFenceWrapperException(ErrorType errorType) {
        this.errorType = errorType;
    }

    public GeoFenceWrapperException(ErrorType errorType, Exception e) {
        super(e);
        this.errorType = errorType;
    }

    public ErrorType getErrorType() {
        return errorType;
    }
}
